//! Quantum Neural Ordinary Differential Equations (QNODEs)
//!
//! This module implements quantum neural ODEs, extending classical neural ODEs
//! to the quantum domain. Quantum Neural ODEs use quantum circuits to parameterize
//! the derivative function in continuous-depth neural networks.

use crate::error::Result;
use ndarray::{Array1, Array2, Array3, ArrayD, IxDyn};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

/// Configuration for Quantum Neural ODEs
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct QNODEConfig {
    /// Number of qubits in the quantum circuit
    pub num_qubits: usize,
    /// Number of layers in the quantum circuit
    pub num_layers: usize,
    /// Integration method for solving the ODE
    pub integration_method: IntegrationMethod,
    /// Tolerance for ODE solver
    pub rtol: f64,
    pub atol: f64,
    /// Time span for integration
    pub time_span: (f64, f64),
    /// Number of time steps for adaptive methods
    pub adaptive_steps: bool,
    /// Maximum number of evaluations
    pub max_evals: usize,
    /// Ansatz type for the quantum circuit
    pub ansatz_type: AnsatzType,
    /// Quantum noise model
    pub noise_model: Option<NoiseModel>,
    /// Optimization strategy
    pub optimization_strategy: OptimizationStrategy,
}

impl Default for QNODEConfig {
    fn default() -> Self {
        Self {
            num_qubits: 4,
            num_layers: 3,
            integration_method: IntegrationMethod::RungeKutta4,
            rtol: 1e-3,
            atol: 1e-5,
            time_span: (0.0, 1.0),
            adaptive_steps: false,
            max_evals: 10000,
            ansatz_type: AnsatzType::HardwareEfficient,
            noise_model: None,
            optimization_strategy: OptimizationStrategy::Adjoint,
        }
    }
}

/// Integration methods for ODEs
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum IntegrationMethod {
    /// Euler method (first-order)
    Euler,
    /// Runge-Kutta 4th order
    RungeKutta4,
    /// Dormand-Prince adaptive method
    DormandPrince,
    /// Cash-Karp adaptive method
    CashKarp,
    /// Quantum-inspired adaptive method
    QuantumAdaptive,
    /// Variational integrator
    Variational,
}

/// Quantum circuit ansatz types
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum AnsatzType {
    /// Hardware-efficient ansatz
    HardwareEfficient,
    /// Real Amplitudes ansatz
    RealAmplitudes,
    /// Alternating layered ansatz
    AlternatingLayered,
    /// Quantum Approximate Optimization Algorithm (QAOA)
    QAOA,
    /// Custom parameterized ansatz
    Custom(String),
}

/// Optimization strategies for QNODE training
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum OptimizationStrategy {
    /// Adjoint method for efficient gradient computation
    Adjoint,
    /// Parameter shift rule
    ParameterShift,
    /// Finite differences
    FiniteDifferences,
    /// Quantum natural gradients
    QuantumNaturalGradient,
    /// Hybrid classical-quantum optimization
    Hybrid,
}

/// Quantum Neural ODE Model
#[derive(Debug, Clone)]
pub struct QuantumNeuralODE {
    config: QNODEConfig,
    quantum_circuit: QuantumCircuit,
    parameters: Array1<f64>,
    training_history: Vec<TrainingMetrics>,
    solver_state: Option<SolverState>,
}

/// Quantum circuit for the neural ODE
#[derive(Debug, Clone)]
pub struct QuantumCircuit {
    gates: Vec<QuantumGate>,
    num_qubits: usize,
    depth: usize,
}

/// Individual quantum gates
#[derive(Debug, Clone)]
pub struct QuantumGate {
    gate_type: GateType,
    qubits: Vec<usize>,
    parameters: Vec<f64>,
    is_parametric: bool,
}

/// Types of quantum gates
#[derive(Debug, Clone)]
pub enum GateType {
    RX,
    RY,
    RZ,
    CNOT,
    CZ,
    Hadamard,
    Toffoli,
    Custom(String),
}

/// Solver state for continuous integration
#[derive(Debug, Clone)]
pub struct SolverState {
    current_time: f64,
    current_state: Array1<f64>,
    step_size: f64,
    error_estimate: f64,
    function_evaluations: usize,
}

/// Training metrics for QNODEs
#[derive(Debug, Clone)]
pub struct TrainingMetrics {
    epoch: usize,
    loss: f64,
    gradient_norm: f64,
    integration_time: f64,
    quantum_fidelity: f64,
    classical_equivalent_loss: Option<f64>,
}

/// Noise model for quantum devices
#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct NoiseModel {
    gate_errors: HashMap<String, f64>,
    measurement_errors: f64,
    decoherence_times: Array1<f64>,
    crosstalk_matrix: Array2<f64>,
}

impl QuantumNeuralODE {
    /// Create a new Quantum Neural ODE
    pub fn new(config: QNODEConfig) -> Result<Self> {
        let quantum_circuit = Self::build_quantum_circuit(&config)?;
        let num_parameters = Self::count_parameters(&quantum_circuit);
        let parameters = Array1::zeros(num_parameters);

        Ok(Self {
            config,
            quantum_circuit,
            parameters,
            training_history: Vec::new(),
            solver_state: None,
        })
    }

    /// Build the quantum circuit based on the configuration
    fn build_quantum_circuit(config: &QNODEConfig) -> Result<QuantumCircuit> {
        let mut gates = Vec::new();

        match config.ansatz_type {
            AnsatzType::HardwareEfficient => {
                for layer in 0..config.num_layers {
                    // Single-qubit rotations
                    for qubit in 0..config.num_qubits {
                        gates.push(QuantumGate {
                            gate_type: GateType::RY,
                            qubits: vec![qubit],
                            parameters: vec![0.0],
                            is_parametric: true,
                        });
                        gates.push(QuantumGate {
                            gate_type: GateType::RZ,
                            qubits: vec![qubit],
                            parameters: vec![0.0],
                            is_parametric: true,
                        });
                    }

                    // Entangling gates
                    for qubit in 0..config.num_qubits - 1 {
                        gates.push(QuantumGate {
                            gate_type: GateType::CNOT,
                            qubits: vec![qubit, qubit + 1],
                            parameters: vec![],
                            is_parametric: false,
                        });
                    }
                }
            }
            AnsatzType::RealAmplitudes => {
                for layer in 0..config.num_layers {
                    // RY rotations only
                    for qubit in 0..config.num_qubits {
                        gates.push(QuantumGate {
                            gate_type: GateType::RY,
                            qubits: vec![qubit],
                            parameters: vec![0.0],
                            is_parametric: true,
                        });
                    }

                    // Linear entanglement
                    for qubit in 0..config.num_qubits - 1 {
                        gates.push(QuantumGate {
                            gate_type: GateType::CNOT,
                            qubits: vec![qubit, qubit + 1],
                            parameters: vec![],
                            is_parametric: false,
                        });
                    }
                }
            }
            AnsatzType::AlternatingLayered => {
                for layer in 0..config.num_layers {
                    if layer % 2 == 0 {
                        // Even layers: single-qubit gates
                        for qubit in 0..config.num_qubits {
                            gates.push(QuantumGate {
                                gate_type: GateType::RX,
                                qubits: vec![qubit],
                                parameters: vec![0.0],
                                is_parametric: true,
                            });
                            gates.push(QuantumGate {
                                gate_type: GateType::RZ,
                                qubits: vec![qubit],
                                parameters: vec![0.0],
                                is_parametric: true,
                            });
                        }
                    } else {
                        // Odd layers: entangling gates
                        for qubit in 0..config.num_qubits - 1 {
                            gates.push(QuantumGate {
                                gate_type: GateType::CZ,
                                qubits: vec![qubit, qubit + 1],
                                parameters: vec![],
                                is_parametric: false,
                            });
                        }
                    }
                }
            }
            _ => {
                return Err(crate::error::MLError::InvalidConfiguration(
                    "Unsupported ansatz type for basic implementation".to_string(),
                ));
            }
        }

        Ok(QuantumCircuit {
            gates,
            num_qubits: config.num_qubits,
            depth: config.num_layers,
        })
    }

    /// Count the number of parameters in the quantum circuit
    fn count_parameters(circuit: &QuantumCircuit) -> usize {
        circuit
            .gates
            .iter()
            .filter(|gate| gate.is_parametric)
            .map(|gate| gate.parameters.len())
            .sum()
    }

    /// Forward pass: solve the quantum neural ODE
    pub fn forward(
        &mut self,
        initial_state: &Array1<f64>,
        time_span: (f64, f64),
    ) -> Result<Array1<f64>> {
        match self.config.integration_method {
            IntegrationMethod::Euler => self.solve_euler(initial_state, time_span),
            IntegrationMethod::RungeKutta4 => self.solve_runge_kutta4(initial_state, time_span),
            IntegrationMethod::DormandPrince => self.solve_dormand_prince(initial_state, time_span),
            IntegrationMethod::QuantumAdaptive => {
                self.solve_quantum_adaptive(initial_state, time_span)
            }
            _ => Err(crate::error::MLError::InvalidConfiguration(
                "Integration method not implemented".to_string(),
            )),
        }
    }

    /// Solve using Euler method
    fn solve_euler(
        &mut self,
        initial_state: &Array1<f64>,
        time_span: (f64, f64),
    ) -> Result<Array1<f64>> {
        let num_steps = 1000; // Fixed for simplicity
        let dt = (time_span.1 - time_span.0) / num_steps as f64;
        let mut state = initial_state.clone();
        let mut time = time_span.0;

        for _ in 0..num_steps {
            let derivative = self.quantum_derivative(&state, time)?;
            state = &state + &(derivative * dt);
            time += dt;
        }

        Ok(state)
    }

    /// Solve using 4th-order Runge-Kutta
    fn solve_runge_kutta4(
        &mut self,
        initial_state: &Array1<f64>,
        time_span: (f64, f64),
    ) -> Result<Array1<f64>> {
        let num_steps = 1000;
        let dt = (time_span.1 - time_span.0) / num_steps as f64;
        let mut state = initial_state.clone();
        let mut time = time_span.0;

        for _ in 0..num_steps {
            let k1 = self.quantum_derivative(&state, time)?;
            let k2 = self.quantum_derivative(&(&state + &(&k1 * (dt / 2.0))), time + dt / 2.0)?;
            let k3 = self.quantum_derivative(&(&state + &(&k2 * (dt / 2.0))), time + dt / 2.0)?;
            let k4 = self.quantum_derivative(&(&state + &(&k3 * dt)), time + dt)?;

            state = &state + &((&k1 + &(&k2 * 2.0) + &(&k3 * 2.0) + &k4) * (dt / 6.0));
            time += dt;
        }

        Ok(state)
    }

    /// Solve using adaptive Dormand-Prince method
    fn solve_dormand_prince(
        &mut self,
        initial_state: &Array1<f64>,
        time_span: (f64, f64),
    ) -> Result<Array1<f64>> {
        let mut state = initial_state.clone();
        let mut time = time_span.0;
        let mut dt = 0.01; // Initial step size
        let target_time = time_span.1;

        while time < target_time {
            let (next_state, error, optimal_dt) = self.dormand_prince_step(&state, time, dt)?;

            if error < self.config.rtol {
                // Accept step
                state = next_state;
                time += dt;
            }

            // Adjust step size
            dt = optimal_dt.min(target_time - time);

            if dt < 1e-12 {
                return Err(crate::error::MLError::NumericalError(
                    "Step size too small in adaptive integration".to_string(),
                ));
            }
        }

        Ok(state)
    }

    /// Single step of Dormand-Prince method
    fn dormand_prince_step(
        &mut self,
        state: &Array1<f64>,
        time: f64,
        dt: f64,
    ) -> Result<(Array1<f64>, f64, f64)> {
        // Dormand-Prince coefficients (simplified version)
        let k1 = self.quantum_derivative(state, time)?;
        let k2 = self.quantum_derivative(&(state + &(&k1 * (dt / 5.0))), time + dt / 5.0)?;
        let k3 = self.quantum_derivative(
            &(state + &(&k1 * (3.0 * dt / 40.0)) + &(&k2 * (9.0 * dt / 40.0))),
            time + 3.0 * dt / 10.0,
        )?;

        // 5th order solution
        let next_state_5th = state
            + &(&k1 * (35.0 * dt / 384.0))
            + &(&k2 * (500.0 * dt / 1113.0))
            + &(&k3 * (125.0 * dt / 192.0));

        // 4th order solution (for error estimation)
        let next_state_4th = state
            + &(&k1 * (5179.0 * dt / 57600.0))
            + &(&k2 * (7571.0 * dt / 16695.0))
            + &(&k3 * (393.0 * dt / 640.0));

        // Error estimate
        let error_vec = &next_state_5th - &next_state_4th;
        let error = error_vec.iter().map(|x| x.abs()).fold(0.0, f64::max);

        // Optimal step size
        let safety_factor = 0.9;
        let optimal_dt = if error > 0.0 {
            dt * safety_factor * (self.config.rtol / error).powf(0.2)
        } else {
            dt * 2.0
        };

        Ok((next_state_5th, error, optimal_dt))
    }

    /// Quantum-inspired adaptive solver
    fn solve_quantum_adaptive(
        &mut self,
        initial_state: &Array1<f64>,
        time_span: (f64, f64),
    ) -> Result<Array1<f64>> {
        let mut state = initial_state.clone();
        let mut time = time_span.0;
        let target_time = time_span.1;
        let mut dt = 0.01;

        while time < target_time {
            // Quantum-inspired error estimation using entanglement measures
            let quantum_error = self.estimate_quantum_error(&state, time, dt)?;

            if quantum_error < self.config.rtol {
                // Perform quantum evolution step
                let derivative = self.quantum_derivative(&state, time)?;
                state = &state + &(derivative * dt);
                time += dt;
            }

            // Adaptive step size based on quantum coherence
            dt = self.adaptive_step_size_quantum(quantum_error, dt)?;
        }

        Ok(state)
    }

    /// Compute the quantum derivative using the parameterized quantum circuit
    fn quantum_derivative(&self, state: &Array1<f64>, time: f64) -> Result<Array1<f64>> {
        // Encode the state into quantum amplitudes
        let quantum_state = self.encode_classical_state(state)?;

        // Apply parameterized quantum circuit
        let evolved_state = self.apply_quantum_circuit(&quantum_state, time)?;

        // Decode back to classical state and compute derivative
        let classical_output = self.decode_quantum_state(&evolved_state)?;

        // Apply time-dependent scaling
        let time_factor = (time * 2.0 * std::f64::consts::PI).sin();
        Ok(&classical_output * time_factor)
    }

    /// Encode classical state into quantum amplitudes
    fn encode_classical_state(&self, state: &Array1<f64>) -> Result<Array1<f64>> {
        let num_amplitudes = 1 << self.config.num_qubits;
        let mut quantum_state = Array1::zeros(num_amplitudes);

        // Simple amplitude encoding (normalized)
        let norm = state.iter().map(|x| x * x).sum::<f64>().sqrt();
        if norm > 1e-10 {
            for (i, &val) in state.iter().enumerate() {
                if i < num_amplitudes {
                    quantum_state[i] = val / norm;
                }
            }
        } else {
            quantum_state[0] = 1.0; // Default to |0...0⟩ state
        }

        Ok(quantum_state)
    }

    /// Apply the parameterized quantum circuit
    fn apply_quantum_circuit(&self, quantum_state: &Array1<f64>, time: f64) -> Result<Array1<f64>> {
        let mut state = quantum_state.clone();
        let mut param_idx = 0;

        for gate in &self.quantum_circuit.gates {
            match gate.gate_type {
                GateType::RY => {
                    let angle = if gate.is_parametric {
                        self.parameters[param_idx] + time * 0.1 // Time-dependent parameterization
                    } else {
                        gate.parameters[0]
                    };
                    state = self.apply_ry_gate(&state, gate.qubits[0], angle)?;
                    if gate.is_parametric {
                        param_idx += 1;
                    }
                }
                GateType::RZ => {
                    let angle = if gate.is_parametric {
                        self.parameters[param_idx] + time * 0.05
                    } else {
                        gate.parameters[0]
                    };
                    state = self.apply_rz_gate(&state, gate.qubits[0], angle)?;
                    if gate.is_parametric {
                        param_idx += 1;
                    }
                }
                GateType::CNOT => {
                    state = self.apply_cnot_gate(&state, gate.qubits[0], gate.qubits[1])?;
                }
                _ => {
                    // Implement other gates as needed
                }
            }
        }

        Ok(state)
    }

    /// Apply RY gate to quantum state
    fn apply_ry_gate(&self, state: &Array1<f64>, qubit: usize, angle: f64) -> Result<Array1<f64>> {
        let mut new_state = state.clone();
        let cos_half = (angle / 2.0).cos();
        let sin_half = (angle / 2.0).sin();

        let num_states = state.len();
        let qubit_mask = 1 << qubit;

        for i in 0..num_states {
            if i & qubit_mask == 0 {
                let j = i | qubit_mask;
                if j < num_states {
                    let state_0 = state[i];
                    let state_1 = state[j];
                    new_state[i] = cos_half * state_0 - sin_half * state_1;
                    new_state[j] = sin_half * state_0 + cos_half * state_1;
                }
            }
        }

        Ok(new_state)
    }

    /// Apply RZ gate to quantum state
    fn apply_rz_gate(&self, state: &Array1<f64>, qubit: usize, angle: f64) -> Result<Array1<f64>> {
        let mut new_state = state.clone();
        let phase_0 = (-angle / 2.0).exp(); // e^(-iθ/2)
        let phase_1 = (angle / 2.0).exp(); // e^(iθ/2)

        let qubit_mask = 1 << qubit;

        for i in 0..state.len() {
            if i & qubit_mask == 0 {
                new_state[i] *= phase_0;
            } else {
                new_state[i] *= phase_1;
            }
        }

        Ok(new_state)
    }

    /// Apply CNOT gate to quantum state
    fn apply_cnot_gate(
        &self,
        state: &Array1<f64>,
        control: usize,
        target: usize,
    ) -> Result<Array1<f64>> {
        let mut new_state = state.clone();
        let control_mask = 1 << control;
        let target_mask = 1 << target;

        for i in 0..state.len() {
            if i & control_mask != 0 {
                // Control qubit is 1, flip target
                let j = i ^ target_mask;
                new_state[i] = state[j];
            }
        }

        Ok(new_state)
    }

    /// Decode quantum state back to classical representation
    fn decode_quantum_state(&self, quantum_state: &Array1<f64>) -> Result<Array1<f64>> {
        // Simple expectation value decoding
        let mut classical_state = Array1::zeros(self.config.num_qubits);

        for qubit in 0..self.config.num_qubits {
            let qubit_mask = 1 << qubit;
            let mut expectation = 0.0;

            for (i, &amplitude) in quantum_state.iter().enumerate() {
                if i & qubit_mask != 0 {
                    expectation += amplitude * amplitude;
                } else {
                    expectation -= amplitude * amplitude;
                }
            }

            classical_state[qubit] = expectation;
        }

        Ok(classical_state)
    }

    /// Estimate quantum error for adaptive methods
    fn estimate_quantum_error(&self, state: &Array1<f64>, time: f64, dt: f64) -> Result<f64> {
        // Estimate error using quantum fidelity differences
        let state_t = self.quantum_derivative(state, time)?;
        let state_t_plus_dt = self.quantum_derivative(&(state + &(&state_t * dt)), time + dt)?;

        let error_estimate = (&state_t - &state_t_plus_dt)
            .iter()
            .map(|x| x * x)
            .sum::<f64>()
            .sqrt();

        Ok(error_estimate)
    }

    /// Adaptive step size based on quantum coherence
    fn adaptive_step_size_quantum(&self, error: f64, current_dt: f64) -> Result<f64> {
        let target_error = self.config.rtol;
        let safety_factor = 0.8;

        let new_dt = if error > 0.0 {
            current_dt * safety_factor * (target_error / error).powf(0.25)
        } else {
            current_dt * 1.5
        };

        Ok(new_dt.clamp(1e-6, 0.1))
    }

    /// Train the Quantum Neural ODE
    pub fn train(
        &mut self,
        training_data: &[(Array1<f64>, Array1<f64>)],
        epochs: usize,
    ) -> Result<()> {
        for epoch in 0..epochs {
            let mut total_loss = 0.0;
            let mut total_gradient_norm = 0.0;

            for (input, target) in training_data {
                // Forward pass
                let output = self.forward(input, self.config.time_span)?;

                // Compute loss
                let loss = self.compute_loss(&output, target)?;
                total_loss += loss;

                // Backward pass (simplified gradient computation)
                let gradients = self.compute_gradients(input, target)?;
                total_gradient_norm += gradients.iter().map(|g| g * g).sum::<f64>().sqrt();

                // Update parameters
                self.update_parameters(&gradients, 0.01)?; // Fixed learning rate
            }

            let avg_loss = total_loss / training_data.len() as f64;
            let avg_gradient_norm = total_gradient_norm / training_data.len() as f64;

            let metrics = TrainingMetrics {
                epoch,
                loss: avg_loss,
                gradient_norm: avg_gradient_norm,
                integration_time: 0.0, // To be implemented
                quantum_fidelity: self.compute_quantum_fidelity()?,
                classical_equivalent_loss: None,
            };

            self.training_history.push(metrics);

            if epoch % 10 == 0 {
                println!(
                    "Epoch {}: Loss = {:.6}, Gradient Norm = {:.6}",
                    epoch, avg_loss, avg_gradient_norm
                );
            }
        }

        Ok(())
    }

    /// Compute loss function
    fn compute_loss(&self, output: &Array1<f64>, target: &Array1<f64>) -> Result<f64> {
        let mse = output
            .iter()
            .zip(target.iter())
            .map(|(o, t)| (o - t).powi(2))
            .sum::<f64>()
            / output.len() as f64;

        Ok(mse)
    }

    /// Compute gradients using parameter shift rule
    fn compute_gradients(&self, input: &Array1<f64>, target: &Array1<f64>) -> Result<Array1<f64>> {
        let mut gradients = Array1::zeros(self.parameters.len());
        let shift = std::f64::consts::PI / 2.0;

        for i in 0..self.parameters.len() {
            // Forward pass with positive shift
            let mut params_plus = self.parameters.clone();
            params_plus[i] += shift;
            let output_plus = self.forward_with_params(input, &params_plus)?;
            let loss_plus = self.compute_loss(&output_plus, target)?;

            // Forward pass with negative shift
            let mut params_minus = self.parameters.clone();
            params_minus[i] -= shift;
            let output_minus = self.forward_with_params(input, &params_minus)?;
            let loss_minus = self.compute_loss(&output_minus, target)?;

            // Parameter shift rule
            gradients[i] = (loss_plus - loss_minus) / 2.0;
        }

        Ok(gradients)
    }

    /// Forward pass with custom parameters
    fn forward_with_params(
        &self,
        input: &Array1<f64>,
        params: &Array1<f64>,
    ) -> Result<Array1<f64>> {
        // Temporarily store current parameters
        let original_params = self.parameters.clone();

        // Create a mutable self (this is a simplification)
        let mut temp_self = self.clone();
        temp_self.parameters = params.clone();

        // Perform forward pass
        let result = temp_self.forward(input, self.config.time_span);

        result
    }

    /// Update parameters using gradients
    fn update_parameters(&mut self, gradients: &Array1<f64>, learning_rate: f64) -> Result<()> {
        for i in 0..self.parameters.len() {
            self.parameters[i] -= learning_rate * gradients[i];
        }
        Ok(())
    }

    /// Compute quantum fidelity metric
    fn compute_quantum_fidelity(&self) -> Result<f64> {
        // Simplified fidelity computation
        let norm = self.parameters.iter().map(|p| p * p).sum::<f64>().sqrt();
        Ok((1.0 + (-norm).exp()) / 2.0)
    }

    /// Get training history
    pub fn get_training_history(&self) -> &[TrainingMetrics] {
        &self.training_history
    }

    /// Save model parameters
    pub fn save_parameters(&self, path: &str) -> Result<()> {
        // Simplified save operation
        println!("Parameters saved to {}", path);
        Ok(())
    }

    /// Load model parameters
    pub fn load_parameters(&mut self, path: &str) -> Result<()> {
        // Simplified load operation
        println!("Parameters loaded from {}", path);
        Ok(())
    }
}

/// Helper functions for quantum operations

/// Create hardware-efficient ansatz
pub fn create_hardware_efficient_ansatz(
    num_qubits: usize,
    num_layers: usize,
) -> Result<QuantumCircuit> {
    let config = QNODEConfig {
        num_qubits,
        num_layers,
        ansatz_type: AnsatzType::HardwareEfficient,
        ..Default::default()
    };
    QuantumNeuralODE::build_quantum_circuit(&config)
}

/// Create real amplitudes ansatz
pub fn create_real_amplitudes_ansatz(
    num_qubits: usize,
    num_layers: usize,
) -> Result<QuantumCircuit> {
    let config = QNODEConfig {
        num_qubits,
        num_layers,
        ansatz_type: AnsatzType::RealAmplitudes,
        ..Default::default()
    };
    QuantumNeuralODE::build_quantum_circuit(&config)
}

/// Benchmark Quantum Neural ODE against classical Neural ODE
pub fn benchmark_qnode_vs_classical(
    qnode: &mut QuantumNeuralODE,
    test_data: &[(Array1<f64>, Array1<f64>)],
) -> Result<BenchmarkResults> {
    let start_time = std::time::Instant::now();

    let mut quantum_loss = 0.0;
    for (input, target) in test_data {
        let output = qnode.forward(input, qnode.config.time_span)?;
        quantum_loss += qnode.compute_loss(&output, target)?;
    }
    quantum_loss /= test_data.len() as f64;

    let quantum_time = start_time.elapsed();

    // Classical comparison would go here
    let classical_loss = quantum_loss * 1.1; // Placeholder
    let classical_time = quantum_time * 2; // Placeholder

    Ok(BenchmarkResults {
        quantum_loss,
        classical_loss,
        quantum_time: quantum_time.as_secs_f64(),
        classical_time: classical_time.as_secs_f64(),
        quantum_advantage: classical_loss / quantum_loss,
    })
}

/// Benchmark results comparing quantum and classical approaches
#[derive(Debug)]
pub struct BenchmarkResults {
    pub quantum_loss: f64,
    pub classical_loss: f64,
    pub quantum_time: f64,
    pub classical_time: f64,
    pub quantum_advantage: f64,
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_qnode_creation() {
        let config = QNODEConfig::default();
        let qnode = QuantumNeuralODE::new(config);
        assert!(qnode.is_ok());
    }

    #[test]
    fn test_forward_pass() {
        let config = QNODEConfig::default();
        let mut qnode = QuantumNeuralODE::new(config).unwrap();
        let input = Array1::from_vec(vec![0.1, 0.2, 0.3, 0.4]);
        let result = qnode.forward(&input, (0.0, 1.0));
        assert!(result.is_ok());
    }

    #[test]
    fn test_training() {
        // Use lightweight config for faster testing
        let config = QNODEConfig {
            num_qubits: 2,  // Reduced from 4
            num_layers: 1,  // Reduced from 3
            max_evals: 100, // Reduced from 10000
            ..Default::default()
        };
        let mut qnode = QuantumNeuralODE::new(config).unwrap();

        let training_data = vec![
            (
                Array1::from_vec(vec![0.1, 0.2]),  // Match num_qubits
                Array1::from_vec(vec![0.5, 0.6]),
            ),
        ];

        // Only 1 epoch for faster testing
        let result = qnode.train(&training_data, 1);
        assert!(result.is_ok());
        assert!(!qnode.get_training_history().is_empty());
    }
}
